@@ -5,16 +5,48 @@ sdl_config = with_config('sdl-config', 'sdl-config')
 $CFLAGS += ' ' + `#{sdl_config} --cflags`.chomp
 $LOCAL_LIBS += ' ' + `#{sdl_config} --libs`.chomp
 
-have_func("TTF_OpenFontIndex","SDL_ttf.h")
-have_func("TTF_FontFaces","SDL_ttf.h")
-have_func("TTF_FontFaceIsFixedWidth","SDL_ttf.h")
-have_func("TTF_FontFaceFamilyName","SDL_ttf.h")
-have_func("TTF_FontFaceStyleName","SDL_ttf.h")
+if /-Dmain=SDL_main/ =~ $CFLAGS then
+  def try_func(func, libs, headers = nil, &b)
+    headers = cpp_include(headers)
+    try_link(<<"SRC", libs, &b) or try_link(<<"SRC", libs, &b)
+#{headers}
+/*top*/
+int main(int argc,char** argv) { return 0; }
+int t() { #{func}(); return 0; }
+SRC
+#{COMMON_HEADERS}
+#{headers}
+/*top*/
+int main(int argc,char** argv) { return 0; }
+int t() { void ((*volatile p)()); p = (void ((*)()))#{func}; return 0; }
+SRC
+  end
+end
+
+
+if enable_config("static-libs",false) then
+  have_library("stdc++")
+  have_library("z")
+  have_library("png")
+  have_library("tiff")
+  have_library("jpeg")
+  have_library("freetype")
+  have_library("iconv")
+  have_library("ogg")
+  have_library("vorbis")
+  have_library("vorbisfile")
+  have_library("vorbisenc")
+  have_library("winmm")
+  have_library("SDL")
+end
 
 if enable_config("event2",true) then
   $CFLAGS+= " -D DEF_EVENT2"
 end
 
+if have_library("smpeg","SMPEG_new") then
+  $CFLAGS+= " -D HAVE_SMPEG "
+end
 if have_library("SDL_mixer","Mix_OpenAudio") then
   $CFLAGS+= " -D HAVE_SDL_MIXER "
 end
@@ -27,13 +59,17 @@ end
 if have_library("SDL_ttf","TTF_Init") then
   $CFLAGS+= " -D HAVE_SDL_TTF "
 end
-if have_library("smpeg","SMPEG_new") then
-  $CFLAGS+= " -D HAVE_SMPEG "
-end
+
+have_func("TTF_OpenFontIndex","SDL_ttf.h")
+have_func("TTF_FontFaces","SDL_ttf.h")
+have_func("TTF_FontFaceIsFixedWidth","SDL_ttf.h")
+have_func("TTF_FontFaceFamilyName","SDL_ttf.h")
+have_func("TTF_FontFaceStyleName","SDL_ttf.h")
+
 if have_library("SDLSKK","SDLSKK_Context_new") then
   $CFLAGS+= " -D HAVE_SDLSKK "
 end
-if enable_config("opengl",false) then
+if enable_config("opengl",true) then
   dir_config('x11','/usr/X11R6')
   
   $CFLAGS+= " -D DEF_OPENGL "
