@@ -167,7 +167,10 @@ module SDL
       alias draw_aa_circle_alpha drawAACircleAlpha
       alias draw_aa_ellipse_alpha drawAAEllipseAlpha
 
-      
+      alias draw_bezier drawBezier
+      alias draw_aa_bezier drawAABezier
+      alias draw_bezier_alpha drawBezierAlpha
+      alias draw_aa_bezier_alpha drawAABezierAlpha
     end
   end
 
@@ -187,6 +190,7 @@ module SDL
   if defined?(BMFont) then
     class BMFont
       alias set_color setColor
+      alias text_size textSize
     end
   end
 
@@ -210,6 +214,8 @@ module SDL
   class << CD
     alias num_drive numDrive
     alias index_name indexName
+    alias frames_to_msf framesToMSF
+    alias msf_to_frames MSFToFrames
 # alias open open
   end
 
@@ -272,6 +278,7 @@ class << SDL::WM
   alias set_caption setCaption
 # alias icon= icon=
 # alias iconify iconify
+  alias grab_input grabInput
 end
 
 # rubysdl_mixer.c
@@ -281,6 +288,9 @@ module SDL
 #     alias open open
 #     alias spec spec
       alias play_channel playChannel
+      alias play_channel_timed playChannelTimed
+      alias fade_in_channel fadeInChannel
+      alias fade_in_channel_timed fadeInChannelTimed
 #     alias play? play?
       alias set_volume setVolume
       alias allocate_channels allocateChannels
@@ -288,6 +298,7 @@ module SDL
 #     alias pause pause
 #     alias resume resume
 #     alias pause? pause?
+      alias fade_out fadeOut
       alias play_music playMusic
       alias fade_in_music fadeInMusic
       alias set_volume_music setVolumeMusic
@@ -298,6 +309,7 @@ module SDL
       alias rewind_music rewindMusic
       alias pause_music? pauseMusic?
       alias play_music? playMusic?
+      alias fading_music fadingMusic
     end
 
     module Mixer
@@ -368,9 +380,18 @@ module SDL
   end
 end
 
+# rubysdl_opengl.c
+class <<  SDL
+  if method_defined?(:getGLAttr)
+    alias get_GL_attr getGLAttr
+    alias set_GL_attr setGLAttr
+    alias GL_swap_buffers GLSwapBuffers
+  end
+end
+
 # sdl.rb
 
-module  SDL
+module SDL
   class Surface
     if method_defined?(:rotateScaledSurface) then
       alias rotate_scaled_surface rotateScaledSurface
