@@ -1,6 +1,6 @@
 #
 # Ruby/SDL   Ruby extension library for SDL
-#  Copyright (C) 2001,2002 Ohbayashi Ippei
+#  Copyright (C) 2001-2005 Ohbayashi Ippei
 # 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -23,7 +23,7 @@ end
 
 module SDL
 
-  VERSION = "0.9.3"
+  VERSION = "1.0.0"
 
   class PixelFormat
 
@@ -36,7 +36,7 @@ module SDL
     def_delegators( :@surface, :mapRGB, :mapRGBA, :getRGB, :getRGBA, :bpp,
 		    :colorkey, :alpha )
   end
-  
+
   class Surface
     def put(surface,x,y)
       SDL::blitSurface(surface,0,0,surface.w,surface.h,self,x,y)
@@ -55,7 +55,13 @@ module SDL
     
     def copyRect(x,y,w,h)
       flagbase=SDL::SWSURFACE|SDL::HWSURFACE|SDL::SRCCOLORKEY|SDL::SRCALPHA
-      new_surface=Surface.new(flagbase&self.flags,w,h,self)
+      alpha_flag = self.flags & (SDL::SRCCOLORKEY|SDL::RLEACCEL)
+      self.setAlpha(0,self.alpha)
+      begin
+        new_surface=Surface.new(flagbase&self.flags,w,h,self)
+      ensure
+        self.setAlpha(alpha_flag,self.alpha)
+      end
       SDL.blitSurface(self,x,y,w,h,new_surface,0,0)
       new_surface.setColorKey(self.flags & (SDL::SRCCOLORKEY|SDL::RLEACCEL),
                               self.colorkey )
@@ -63,7 +69,17 @@ module SDL
                            self.alpha )
       return new_surface
     end
-    
+
+    def self.new(*args)
+      case args.size
+      when 4
+        create(*args)
+      when 8
+        createWithFormat(*args)
+      else
+        raise ArgumentError,"must be SDL::Surface.new(flags,w,h,surface) or SDL::Surface.new(flags,w,h,depth,Rmask,Gmask,Bmask,Amask)"
+      end
+    end
   end
 
   def color2int(color,format)
@@ -124,6 +140,12 @@ module SDL
     end
     
   end # of module Mouse
+
+  class CD
+    def in_drive?
+      status > 0
+    end
+  end
   
   module_function
 
@@ -217,7 +239,7 @@ module SDL
       end
     end
   end
-    
+  
 end
 
 if defined?(GL) then
