@@ -1,7 +1,7 @@
 /*
   Ruby/SDL   Ruby extension library for SDL
 
-  Copyright (C) 2001-2004 Ohbayashi Ippei
+  Copyright (C) 2001-2005 Ohbayashi Ippei
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -296,6 +296,11 @@ void defineConstForKey()
   rb_define_const(mKey,"MOD_SHIFT",UINT2NUM(KMOD_SHIFT));
   rb_define_const(mKey,"MOD_ALT",UINT2NUM(KMOD_ALT));
   rb_define_const(mKey,"MOD_META",UINT2NUM(KMOD_META));
+
+  /* key repeat constants*/
+  rb_define_const(mKey,"DEFAULT_REPEAT_DELAY", INT2NUM(SDL_DEFAULT_REPEAT_DELAY));
+  rb_define_const(mKey,"DEFAULT_REPEAT_INTERVAL", INT2NUM(SDL_DEFAULT_REPEAT_INTERVAL));
+  
 }
 
 /* under 2 function is for testing */
@@ -307,6 +312,9 @@ static VALUE sdl_keyScan(VALUE mod)
 }
 static VALUE sdl_keyPressed(VALUE mod,VALUE keysym)
 {
+  if(NUM2INT(keysym) <SDLK_FIRST || SDLK_LAST < NUM2INT(keysym))
+    rb_raise(eSDLError, "keysym number is out of range");
+  
   return (keyState[NUM2INT(keysym)]==SDL_PRESSED)?Qtrue:Qfalse;
 }
 static VALUE sdl_modState(VALUE mod)
@@ -340,6 +348,9 @@ void init_keyEvent()
   rb_define_module_function(mKey,"enableKeyRepeat",sdl_enableKeyRepeat,2);
   rb_define_module_function(mKey,"disableKeyRepeat",sdl_disableKeyRepeat,0);
   rb_define_module_function(mKey,"getKeyName",sdl_getKeyName,1);
+
+  keyState = malloc(SDLK_LAST*sizeof(Uint8));
+  memset(keyState, SDL_RELEASED, SDLK_LAST);
   
   defineConstForKey();
 }
