@@ -1,7 +1,7 @@
 /*
   Ruby/SDL   Ruby extension library for SDL
 
-  Copyright (C) 2001-2004 Ohbayashi Ippei
+  Copyright (C) 2001-2005 Ohbayashi Ippei
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -230,6 +230,56 @@ static VALUE sdl_drawFilledEllipseAlpha(VALUE obj,VALUE x,VALUE y,VALUE rx,VALUE
   return Qnil;
 }
 
+static VALUE sdl_drawBezier(VALUE obj,VALUE x1,VALUE y1,VALUE x2,VALUE y2,
+                            VALUE x3,VALUE y3,VALUE x4,VALUE y4,
+                            VALUE level,VALUE color)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  sge_Bezier(surface,NUM2INT(x1),NUM2INT(y1),NUM2INT(x2),NUM2INT(y2),
+             NUM2INT(x3),NUM2INT(y3),NUM2INT(x4),NUM2INT(y4),
+             NUM2INT(level),VALUE2COLOR(color,surface->format));
+  return Qnil;
+}
+
+static VALUE sdl_drawBezierAlpha(VALUE obj,VALUE x1,VALUE y1,VALUE x2,VALUE y2,
+                                 VALUE x3,VALUE y3,VALUE x4,VALUE y4,
+                                 VALUE level,VALUE color,VALUE alpha)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  sge_BezierAlpha(surface,NUM2INT(x1),NUM2INT(y1),NUM2INT(x2),NUM2INT(y2),
+                  NUM2INT(x3),NUM2INT(y3),NUM2INT(x4),NUM2INT(y4),
+                  NUM2INT(level),VALUE2COLOR(color,surface->format),
+                  NUM2UINT(alpha));
+  return Qnil;
+}
+
+static VALUE sdl_drawAABezier(VALUE obj,VALUE x1,VALUE y1,VALUE x2,VALUE y2,
+                              VALUE x3,VALUE y3,VALUE x4,VALUE y4,
+                              VALUE level,VALUE color)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  sge_AABezier(surface,NUM2INT(x1),NUM2INT(y1),NUM2INT(x2),NUM2INT(y2),
+               NUM2INT(x3),NUM2INT(y3),NUM2INT(x4),NUM2INT(y4),
+               NUM2INT(level),VALUE2COLOR(color,surface->format));
+  return Qnil;
+}
+
+static VALUE sdl_drawAABezierAlpha(VALUE obj,VALUE x1,VALUE y1,VALUE x2,VALUE y2,
+                                 VALUE x3,VALUE y3,VALUE x4,VALUE y4,
+                                 VALUE level,VALUE color,VALUE alpha)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  sge_AABezierAlpha(surface,NUM2INT(x1),NUM2INT(y1),NUM2INT(x2),NUM2INT(y2),
+                    NUM2INT(x3),NUM2INT(y3),NUM2INT(x4),NUM2INT(y4),
+                    NUM2INT(level),VALUE2COLOR(color,surface->format),
+                    NUM2UINT(alpha));
+  return Qnil;
+}
+
 static VALUE sdl_rotateScaledSurface(VALUE obj,VALUE angle,VALUE scale,VALUE bgcolor)
 {
   SDL_Surface *surface,*result;
@@ -399,7 +449,26 @@ static VALUE sdl_unset_cdata(VALUE obj, VALUE vx, VALUE vy, VALUE vw, VALUE vh)
   return Qnil;
 }
 
+static VALUE sdl_w_cdata(VALUE obj)
+{
+  sge_cdata * cdata = value_to_collision_map(obj);
+  return INT2FIX(cdata->w);
+}
+
+static VALUE sdl_h_cdata(VALUE obj)
+{
+  sge_cdata * cdata = value_to_collision_map(obj);
+  return INT2FIX(cdata->h);
+}
+
 /* bitmap font */
+static void sdl_bf_close(sge_bmpFont* font)
+{
+  if(!rubysdl_is_quit()){
+    sge_BF_CloseFont(font);
+  }
+}
+
 static VALUE sdl_bf_open(VALUE obj, VALUE file, VALUE flags)
 {
   sge_bmpFont* font;
@@ -408,14 +477,14 @@ static VALUE sdl_bf_open(VALUE obj, VALUE file, VALUE flags)
   if( font == NULL )
     rb_raise(eSDLError,"Couldn't open font: %s", GETCSTR(file));
   
-  return Data_Wrap_Struct(cBMFont,0,sge_BF_CloseFont,font);
+  return Data_Wrap_Struct(cBMFont,0,sdl_bf_close,font);
 }
 
 static VALUE sdl_bf_setColor(VALUE obj,VALUE r,VALUE g,VALUE b)
 {
   sge_bmpFont* font;
   Data_Get_Struct(obj,sge_bmpFont,font);
-
+  
   sge_BF_SetColor(font,NUM2UINT(r),NUM2UINT(g),NUM2UINT(b));
   return Qnil;
 }
@@ -434,6 +503,15 @@ static VALUE sdl_bf_getWidth(VALUE obj)
   return INT2FIX(sge_BF_GetWidth(font));
 }
 
+static VALUE sdl_bf_textSize(VALUE obj, VALUE text)
+{
+  sge_bmpFont* font;
+  SDL_Rect rect;
+  Data_Get_Struct(obj,sge_bmpFont,font);
+  rect = sge_BF_TextSize(font, GETCSTR(text));
+  return rb_ary_new3(2, INT2FIX(rect.w), INT2FIX(rect.h));
+}
+
 static VALUE sdl_bf_textout(VALUE obj,VALUE surface,VALUE string,
                             VALUE x, VALUE y)
 {
@@ -482,14 +560,16 @@ void init_sge_video()
   rb_define_method(cSurface,"drawFilledEllispe",sdl_drawFilledEllipse,5);
   rb_define_method(cSurface,"drawEllipse",sdl_drawEllipse,5);
   rb_define_method(cSurface,"drawFilledEllipse",sdl_drawFilledEllipse,5);
-
+  rb_define_method(cSurface,"drawBezier",sdl_drawBezier,10);
+    
   /* antialiased primitive drawing */
   rb_define_method(cSurface,"drawAALine",sdl_drawAALine,5);
   rb_define_method(cSurface,"drawAACircle",sdl_drawAACircle,4);
   rb_define_method(cSurface,"drawAAFilledCircle",sdl_drawAAFilledCircle,4);
   rb_define_method(cSurface,"drawAAEllipse",sdl_drawAAEllipse,5);
   rb_define_method(cSurface,"drawAAFilledEllipse",sdl_drawAAFilledEllipse,5);
-
+  rb_define_method(cSurface,"drawAABezier",sdl_drawAABezier,10);
+    
   /* primitive drawing with alpha */
   rb_define_method(cSurface,"drawLineAlpha",sdl_drawLineAlpha,6);
   rb_define_method(cSurface,"drawRectAlpha",sdl_drawRectAlpha,6);
@@ -498,12 +578,14 @@ void init_sge_video()
   rb_define_method(cSurface,"drawFilledCircleAlpha",sdl_drawFilledCircleAlpha,5);
   rb_define_method(cSurface,"drawEllipseAlpha",sdl_drawEllipseAlpha,6);
   rb_define_method(cSurface,"drawFilledEllipseAlpha",sdl_drawFilledEllipseAlpha,6);
-
+  rb_define_method(cSurface,"drawBezierAlpha",sdl_drawBezierAlpha,11);
+    
   /* antialiased primitive drawing with alpha */
   rb_define_method(cSurface,"drawAALineAlpha",sdl_drawAALineAlpha,6);
   rb_define_method(cSurface,"drawAACircleAlpha",sdl_drawAACircleAlpha,5);
   rb_define_method(cSurface,"drawAAEllipseAlpha",sdl_drawAAEllipseAlpha,6);
-
+  rb_define_method(cSurface,"drawAABezierAlpha",sdl_drawAABezierAlpha,11);
+    
   /* rotation and scaling */
   rb_define_method(cSurface,"rotateScaledSurface",sdl_rotateScaledSurface,3);
   rb_define_module_function(mSDL,"rotateScaledBlit",sdl_rotateScaledBlit,6);
@@ -522,6 +604,8 @@ void init_sge_video()
   rb_define_method(cCollisionMap,"boundingBoxCheck", sdl_boundingBoxCheck, 5);
   rb_define_method(cCollisionMap,"clear", sdl_unset_cdata, 4);
   rb_define_method(cCollisionMap,"set", sdl_set_cdata, 4);
+  rb_define_method(cCollisionMap,"w", sdl_w_cdata, 0);
+  rb_define_method(cCollisionMap,"h", sdl_h_cdata, 0);
 
   /* bitmap font */
   cBMFont = rb_define_class_under(mSDL,"BMFont",rb_cObject);
@@ -530,6 +614,7 @@ void init_sge_video()
   rb_define_method(cBMFont,"setColor",sdl_bf_setColor,3);
   rb_define_method(cBMFont,"height",sdl_bf_getHeight,0);
   rb_define_method(cBMFont,"width",sdl_bf_getWidth,0);
+  rb_define_method(cBMFont,"textSize",sdl_bf_textSize, 1);
   rb_define_method(cBMFont,"textout",sdl_bf_textout,4);
 
   defineConstForSGE();
