@@ -1,7 +1,7 @@
 /*
   Ruby/SDL   Ruby extension library for SDL
 
-  Copyright (C) 2001-2004 Ohbayashi Ippei
+  Copyright (C) 2001-2005 Ohbayashi Ippei
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -29,13 +29,20 @@ void sdl_freeSurface(SDL_Surface* surface)
 Uint32 VALUE2COLOR(VALUE color,SDL_PixelFormat *format)
 {
   if( rb_obj_is_kind_of( color, rb_cArray ) ){
-    if( RARRAY(color)->len != 3 ){
-      rb_raise(rb_eArgError,"type mismatch:color array needs 3 elements");
-    }else{
+    switch( RARRAY(color)->len ){
+    case 3:
       return SDL_MapRGB(format,
 			NUM2UINT(rb_ary_entry(color,0)),
 			NUM2UINT(rb_ary_entry(color,1)),
 			NUM2UINT(rb_ary_entry(color,2)) );
+    case 4:
+      return SDL_MapRGBA(format,
+                         NUM2UINT(rb_ary_entry(color,0)),
+                         NUM2UINT(rb_ary_entry(color,1)),
+                         NUM2UINT(rb_ary_entry(color,2)),
+                         NUM2UINT(rb_ary_entry(color,3)) );
+    default:
+      rb_raise(rb_eArgError,"type mismatch:color array needs 3 or 4 elements");
     }
   }else{
     return NUM2UINT(color);
@@ -217,7 +224,48 @@ static VALUE sdl_createSurface(VALUE class,VALUE flags,VALUE w,VALUE h,
   }
   return Data_Wrap_Struct(class,0,sdl_freeSurface,newSurface);
 }
-				    
+
+static VALUE sdl_createSurfaceWithFormat(VALUE class,VALUE flags,VALUE w,
+                                         VALUE h,VALUE depth,VALUE Rmask,
+                                         VALUE Gmask,VALUE Bmask,
+                                         VALUE Amask)
+{
+  SDL_Surface *surface;
+
+  surface = SDL_CreateRGBSurface(NUM2UINT(flags),NUM2INT(w),NUM2INT(h),
+                                 NUM2UINT(depth),NUM2UINT(Rmask),
+                                 NUM2UINT(Gmask),NUM2UINT(Bmask),
+                                 NUM2UINT(Amask));
+  if( surface == NULL ){
+    rb_raise(eSDLError,"Couldn't Create Surface: %s",SDL_GetError());
+  }
+  return Data_Wrap_Struct(class,0,sdl_freeSurface,surface);
+}
+
+
+static VALUE sdl_createSurfaceFrom(VALUE class,VALUE pixels,VALUE w,
+                                   VALUE h,VALUE depth,VALUE pitch,
+                                   VALUE Rmask,VALUE Gmask,VALUE Bmask,
+                                   VALUE Amask)
+{
+  SDL_Surface *surface;
+  void* pixel_data;
+  
+  StringValue(pixels);
+  pixel_data = malloc(RSTRING(pixels)->len);
+  memcpy(pixel_data,RSTRING(pixels)->ptr,RSTRING(pixels)->len);
+  
+  surface = SDL_CreateRGBSurfaceFrom(pixel_data,NUM2INT(w),NUM2INT(h),
+                                     NUM2UINT(depth),NUM2INT(pitch),
+                                     NUM2UINT(Rmask),NUM2UINT(Gmask),
+                                     NUM2UINT(Bmask),NUM2UINT(Amask));
+  if( surface == NULL ){
+    rb_raise(eSDLError,"Couldn't Create Surface: %s",SDL_GetError());
+  }
+  surface->flags &= ~SDL_PREALLOC;
+  return Data_Wrap_Struct(class,0,sdl_freeSurface,surface);
+}
+
 static VALUE sdl_loadBMP(VALUE class,VALUE filename)
 {
   SDL_Surface *image;
@@ -445,7 +493,7 @@ static VALUE sdl_getPalette(VALUE obj)
   colors = surface->format->palette->colors;
   
   for( i=0; i < surface->format->palette->ncolors; ++i ){
-    color = rb_ary_new3( 3, colors[i].r, colors[i].g, colors[i].b );
+    color = rb_ary_new3( 3, INT2NUM(colors[i].r), INT2NUM(colors[i].g), INT2NUM(colors[i].b) );
     rb_ary_push( palette, color );
   }
   return palette;
@@ -527,6 +575,38 @@ static VALUE sdl_getFlags(VALUE obj)
   Data_Get_Struct(obj,SDL_Surface,surface);
   return UINT2NUM(surface->flags);
 }
+static VALUE sdl_surface_rmask(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->Rmask);
+}
+static VALUE sdl_surface_gmask(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->Gmask);
+}
+static VALUE sdl_surface_bmask(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->Bmask);
+}
+static VALUE sdl_surface_amask(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->Amask);
+}
+
+static VALUE sdl_surface_pixels(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return rb_str_new(surface->pixels,
+                    surface->w * surface->h * surface->format->BytesPerPixel);
+}
 
 static void defineConstForVideo()
 {
@@ -586,7 +666,9 @@ void init_video()
   
   cSurface = rb_define_class_under(mSDL,"Surface",rb_cObject);
 
-  rb_define_singleton_method(cSurface,"new",sdl_createSurface,4);
+  rb_define_singleton_method(cSurface,"create",sdl_createSurface,4);
+  rb_define_singleton_method(cSurface,"createWithFormat",sdl_createSurfaceWithFormat,8);
+  rb_define_singleton_method(cSurface,"new_from",sdl_createSurfaceFrom,9);
   rb_define_singleton_method(cSurface,"loadBMP",sdl_loadBMP,1);
   rb_define_method(cSurface,"saveBMP",sdl_saveBMP,1);
   rb_define_method(cSurface,"displayFormat",sdl_displayFormat,0);
@@ -615,7 +697,12 @@ void init_video()
   rb_define_method(cSurface,"bpp",sdl_getBpp,0);
   rb_define_method(cSurface,"colorkey",sdl_getColorkey,0);
   rb_define_method(cSurface,"alpha",sdl_getAlpha,0);
-
+  rb_define_method(cSurface,"Rmask",sdl_surface_rmask,0);
+  rb_define_method(cSurface,"Gmask",sdl_surface_gmask,0);
+  rb_define_method(cSurface,"Bmask",sdl_surface_bmask,0);
+  rb_define_method(cSurface,"Amask",sdl_surface_amask,0);
+  rb_define_method(cSurface,"pixels",sdl_surface_pixels,0);
+  
   cScreen = rb_define_class_under(mSDL,"Screen",cSurface);
   rb_define_method(cScreen,"updateRect",sdl_updateRect,4);
   rb_define_method(cScreen,"flip",sdl_flip,0);
